@@ -2,7 +2,6 @@ package operator
 
 import (
 	"context"
-	"fmt"
 	"os"
 	"time"
 
@@ -15,12 +14,15 @@ import (
 	"github.com/openshift/cluster-kube-controller-manager-operator/pkg/operator/operatorclient"
 	"github.com/openshift/cluster-kube-controller-manager-operator/pkg/operator/resourcesynccontroller"
 	"github.com/openshift/cluster-kube-controller-manager-operator/pkg/operator/targetconfigcontroller"
+	"github.com/openshift/cluster-kube-controller-manager-operator/pkg/operator/v411_00_assets"
 	"github.com/openshift/library-go/pkg/controller/controllercmd"
 	"github.com/openshift/library-go/pkg/operator/certrotation"
 	"github.com/openshift/library-go/pkg/operator/genericoperatorclient"
+	"github.com/openshift/library-go/pkg/operator/resource/resourceapply"
 	"github.com/openshift/library-go/pkg/operator/staleconditions"
 	"github.com/openshift/library-go/pkg/operator/staticpod"
 	"github.com/openshift/library-go/pkg/operator/staticpod/controller/revision"
+	"github.com/openshift/library-go/pkg/operator/staticresourcecontroller"
 	"github.com/openshift/library-go/pkg/operator/status"
 	"github.com/openshift/library-go/pkg/operator/v1helpers"
 	"k8s.io/apimachinery/pkg/api/errors"
@@ -70,7 +72,36 @@ func RunOperator(ctx context.Context, cc *controllercmd.ControllerContext) error
 		resourceSyncController,
 		cc.EventRecorder,
 	)
+
+	staticResourceController := staticresourcecontroller.NewStaticResourceController(
+		"KubeControllerManagerStaticResources",
+		v411_00_assets.Asset,
+		[]string{
+			"v4.1.0/kube-controller-manager/ns.yaml",
+			"v4.1.0/kube-controller-manager/kubeconfig-cert-syncer.yaml",
+			"v4.1.0/kube-controller-manager/kubeconfig-cm.yaml",
+			"v4.1.0/kube-controller-manager/leader-election-rolebinding.yaml",
+			"v4.1.0/kube-controller-manager/leader-election-cluster-policy-controller-role.yaml",
+			"v4.1.0/kube-controller-manager/leader-election-cluster-policy-controller-rolebinding.yaml",
+			"v4.1.0/kube-controller-manager/leader-election-kube-controller-manager-role-kube-system.yaml",
+			"v4.1.0/kube-controller-manager/leader-election-kube-controller-manager-rolebinding-kube-system.yaml",
+			"v4.1.0/kube-controller-manager/namespace-security-allocation-controller-clusterrole.yaml",
+			"v4.1.0/kube-controller-manager/namespace-security-allocation-controller-clusterrolebinding.yaml",
+			"v4.1.0/kube-controller-manager/svc.yaml",
+			"v4.1.0/kube-controller-manager/sa.yaml",
+			"v4.1.0/kube-controller-manager/localhost-recovery-client-crb.yaml",
+			"v4.1.0/kube-controller-manager/localhost-recovery-sa.yaml",
+			"v4.1.0/kube-controller-manager/localhost-recovery-token.yaml",
+			"v4.1.0/kube-controller-manager/gce/cloud-provider-role.yaml",
+			"v4.1.0/kube-controller-manager/gce/cloud-provider-binding.yaml",
+		},
+		(&resourceapply.ClientHolder{}).WithKubernetes(kubeClient),
+		operatorClient,
+		cc.EventRecorder,
+	).AddKubeInformers(kubeInformersForNamespaces)
+
 	targetConfigController := targetconfigcontroller.NewTargetConfigController(
+		ctx,
 		os.Getenv("IMAGE"),
 		os.Getenv("OPERATOR_IMAGE"),
 		os.Getenv("CLUSTER_POLICY_CONTROLLER_IMAGE"),
@@ -82,7 +113,7 @@ func RunOperator(ctx context.Context, cc *controllercmd.ControllerContext) error
 
 	// don't change any versions until we sync
 	versionRecorder := status.NewVersionGetter()
-	clusterOperator, err := configClient.ConfigV1().ClusterOperators().Get("kube-controller-manager", metav1.GetOptions{})
+	clusterOperator, err := configClient.ConfigV1().ClusterOperators().Get(ctx, "kube-controller-manager", metav1.GetOptions{})
 	if err != nil && !errors.IsNotFound(err) {
 		return err
 	}
@@ -138,12 +169,12 @@ func RunOperator(ctx context.Context, cc *controllercmd.ControllerContext) error
 	if err != nil {
 		return err
 	}
-	saTokenController, err := certrotationcontroller.NewSATokenSignerController(operatorClient, kubeInformersForNamespaces, kubeClient, cc.EventRecorder)
+	saTokenController, err := certrotationcontroller.NewSATokenSignerController(ctx, operatorClient, kubeInformersForNamespaces, kubeClient, cc.EventRecorder)
 	if err != nil {
 		return err
 	}
 
-	staleConditions := staleconditions.NewRemoveStaleConditions(
+	staleConditions := staleconditions.NewRemoveStaleConditionsController(
 		[]string{
 			// the static pod operator used to directly set these. this removes those conditions since the static pod operator was updated.
 			// these can be removed in 4.5
@@ -157,7 +188,8 @@ func RunOperator(ctx context.Context, cc *controllercmd.ControllerContext) error
 	kubeInformersForNamespaces.Start(ctx.Done())
 	dynamicInformers.Start(ctx.Done())
 
-	go staticPodControllers.Run(ctx, 1)
+	go staticPodControllers.Start(ctx)
+	go staticResourceController.Run(ctx, 1)
 	go targetConfigController.Run(1, ctx.Done())
 	go configObserver.Run(ctx, 1)
 	go clusterOperatorStatus.Run(ctx, 1)
@@ -167,7 +199,7 @@ func RunOperator(ctx context.Context, cc *controllercmd.ControllerContext) error
 	go staleConditions.Run(ctx, 1)
 
 	<-ctx.Done()
-	return fmt.Errorf("stopped")
+	return nil
 }
 
 // deploymentConfigMaps is a list of configmaps that are directly copied for the current values.  A different actor/controller modifies these.
