@@ -2,6 +2,7 @@ package targetconfigcontroller
 
 import (
 	"bytes"
+	"context"
 	"crypto/x509"
 	"encoding/json"
 	"fmt"
@@ -45,6 +46,7 @@ import (
 const workQueueKey = "key"
 
 type TargetConfigController struct {
+	ctx                             context.Context
 	targetImagePullSpec             string
 	operatorImagePullSpec           string
 	clusterPolicyControllerPullSpec string
@@ -61,6 +63,7 @@ type TargetConfigController struct {
 }
 
 func NewTargetConfigController(
+	ctx context.Context,
 	targetImagePullSpec, operatorImagePullSpec, clusterPolicyControllerPullSpec string,
 	kubeInformersForNamespaces v1helpers.KubeInformersForNamespaces,
 	operatorClient v1helpers.StaticPodOperatorClient,
@@ -68,6 +71,7 @@ func NewTargetConfigController(
 	eventRecorder events.Recorder,
 ) *TargetConfigController {
 	c := &TargetConfigController{
+		ctx:                             ctx,
 		targetImagePullSpec:             targetImagePullSpec,
 		operatorImagePullSpec:           operatorImagePullSpec,
 		clusterPolicyControllerPullSpec: clusterPolicyControllerPullSpec,
@@ -85,12 +89,9 @@ func NewTargetConfigController(
 	operatorClient.Informer().AddEventHandler(c.eventHandler())
 
 	// these are for watching our outputs in case someone changes them
-	kubeInformersForNamespaces.InformersFor(operatorclient.TargetNamespace).Rbac().V1().Roles().Informer().AddEventHandler(c.eventHandler())
-	kubeInformersForNamespaces.InformersFor(operatorclient.TargetNamespace).Rbac().V1().RoleBindings().Informer().AddEventHandler(c.eventHandler())
 	kubeInformersForNamespaces.InformersFor(operatorclient.TargetNamespace).Core().V1().ConfigMaps().Informer().AddEventHandler(c.eventHandler())
 	kubeInformersForNamespaces.InformersFor(operatorclient.TargetNamespace).Core().V1().Secrets().Informer().AddEventHandler(c.eventHandler())
 	kubeInformersForNamespaces.InformersFor(operatorclient.TargetNamespace).Core().V1().ServiceAccounts().Informer().AddEventHandler(c.eventHandler())
-	kubeInformersForNamespaces.InformersFor(operatorclient.TargetNamespace).Core().V1().Services().Informer().AddEventHandler(c.eventHandler())
 	// we only watch our output namespace
 	kubeInformersForNamespaces.InformersFor(operatorclient.TargetNamespace).Core().V1().Namespaces().Informer().AddEventHandler(c.namespaceEventHandler())
 
@@ -129,7 +130,7 @@ func (c TargetConfigController) sync() error {
 		return err
 	}
 
-	requeue, err := createTargetConfigController(c, c.eventRecorder, operatorSpec)
+	requeue, err := createTargetConfigController(c.ctx, c, c.eventRecorder, operatorSpec)
 	if err != nil {
 		return err
 	}
@@ -175,35 +176,9 @@ func isRequiredConfigPresent(config []byte) error {
 }
 
 // createTargetConfigController takes care of synchronizing (not upgrading) the thing we're managing.
-func createTargetConfigController(c TargetConfigController, recorder events.Recorder, operatorSpec *operatorv1.StaticPodOperatorSpec) (bool, error) {
+func createTargetConfigController(ctx context.Context, c TargetConfigController, recorder events.Recorder, operatorSpec *operatorv1.StaticPodOperatorSpec) (bool, error) {
 	errors := []error{}
 
-	directResourceResults := resourceapply.ApplyDirectly(
-		resourceapply.NewKubeClientHolder(c.kubeClient),
-		c.eventRecorder,
-		v411_00_assets.Asset,
-		"v4.1.0/kube-controller-manager/ns.yaml",
-		"v4.1.0/kube-controller-manager/kubeconfig-cert-syncer.yaml",
-		"v4.1.0/kube-controller-manager/kubeconfig-cm.yaml",
-		"v4.1.0/kube-controller-manager/leader-election-rolebinding.yaml",
-		"v4.1.0/kube-controller-manager/leader-election-cluster-policy-controller-role.yaml",
-		"v4.1.0/kube-controller-manager/leader-election-cluster-policy-controller-rolebinding.yaml",
-		"v4.1.0/kube-controller-manager/leader-election-kube-controller-manager-role-kube-system.yaml",
-		"v4.1.0/kube-controller-manager/leader-election-kube-controller-manager-rolebinding-kube-system.yaml",
-		"v4.1.0/kube-controller-manager/svc.yaml",
-		"v4.1.0/kube-controller-manager/sa.yaml",
-		"v4.1.0/kube-controller-manager/localhost-recovery-client-crb.yaml",
-		"v4.1.0/kube-controller-manager/localhost-recovery-sa.yaml",
-		"v4.1.0/kube-controller-manager/localhost-recovery-token.yaml",
-		"v4.1.0/kube-controller-manager/gce/cloud-provider-role.yaml",
-		"v4.1.0/kube-controller-manager/gce/cloud-provider-binding.yaml",
-	)
-	for _, currResult := range directResourceResults {
-		if currResult.Error != nil {
-			errors = append(errors, fmt.Errorf("%q (%T): %v", currResult.File, currResult.Type, currResult.Error))
-		}
-	}
-
 	_, _, err := manageKubeControllerManagerConfig(c.kubeClient.CoreV1(), recorder, operatorSpec)
 	if err != nil {
 		errors = append(errors, fmt.Errorf("%q: %v", "configmap", err))
@@ -212,7 +187,7 @@ func createTargetConfigController(c TargetConfigController, recorder events.Reco
 	if err != nil {
 		errors = append(errors, fmt.Errorf("%q: %v", "configmap/cluster-policy-controller-config", err))
 	}
-	_, _, err = ManageCSRIntermediateCABundle(c.secretLister, c.kubeClient.CoreV1(), recorder)
+	_, _, err = ManageCSRIntermediateCABundle(ctx, c.secretLister, c.kubeClient.CoreV1(), recorder)
 	if err != nil {
 		errors = append(errors, fmt.Errorf("%q: %v", "configmap/csr-intermediate-ca", err))
 	}
@@ -220,7 +195,7 @@ func createTargetConfigController(c TargetConfigController, recorder events.Reco
 	if err != nil {
 		errors = append(errors, fmt.Errorf("%q: %v", "configmap/csr-controller-ca", err))
 	}
-	_, requeueDelay, _, err := ManageCSRSigner(c.secretLister, c.kubeClient.CoreV1(), recorder)
+	_, requeueDelay, _, err := ManageCSRSigner(ctx, c.secretLister, c.kubeClient.CoreV1(), recorder)
 	if err != nil {
 		errors = append(errors, fmt.Errorf("%q: %v", "secrets/csr-signer", err))
 	}
@@ -231,20 +206,62 @@ func createTargetConfigController(c TargetConfigController, recorder events.Reco
 	if err != nil {
 		errors = append(errors, fmt.Errorf("%q: %v", "configmap/serviceaccount-ca", err))
 	}
-	err = ensureLocalhostRecoverySAToken(c.kubeClient.CoreV1(), recorder)
+	err = ensureLocalhostRecoverySAToken(ctx, c.kubeClient.CoreV1(), recorder)
 	if err != nil {
 		errors = append(errors, fmt.Errorf("%q: %v", "serviceaccount/localhost-recovery-client", err))
 	}
-	_, _, err = managePod(c.kubeClient.CoreV1(), c.kubeClient.CoreV1(), recorder, operatorSpec, c.targetImagePullSpec, c.operatorImagePullSpec, c.clusterPolicyControllerPullSpec)
+
+	// Allow the addition of the service ca to token secrets to be enabled by setting an
+	// UnsupportedConfigOverride field named
+	// enableDeprecatedAndRemovedServiceCAKeyUntilNextRelease_ThisMakesClusterImpossibleToUpgrade
+	// to true.
+	//
+	// This option is provided for backwards-compatibility in 4.5, and should be removed in 4.6.
+	addServingServiceCAToTokenSecrets := false
+	if len(operatorSpec.UnsupportedConfigOverrides.Raw) > 0 {
+		cmConfigOverride := struct {
+			EnableDeprecatedAndRemovedServiceCAKeyUntilNextRelease_ThisMakesClusterImpossibleToUpgrade bool
+		}{}
+		if err := json.Unmarshal(operatorSpec.UnsupportedConfigOverrides.Raw, &cmConfigOverride); err != nil {
+			errors = append(errors, fmt.Errorf("failed to load EnableDeprecatedAndRemovedServiceCAKeyUntilNextRelease_ThisMakesClusterImpossibleToUpgrade from UnsupportedConfigOverride: %v", err))
+		} else {
+			addServingServiceCAToTokenSecrets = cmConfigOverride.EnableDeprecatedAndRemovedServiceCAKeyUntilNextRelease_ThisMakesClusterImpossibleToUpgrade
+		}
+	}
+
+	_, _, err = managePod(ctx, c.kubeClient.CoreV1(), c.kubeClient.CoreV1(), recorder, operatorSpec, c.targetImagePullSpec, c.operatorImagePullSpec, c.clusterPolicyControllerPullSpec, addServingServiceCAToTokenSecrets)
 	if err != nil {
 		errors = append(errors, fmt.Errorf("%q: %v", "configmap/kube-controller-manager-pod", err))
 	}
 
-	err = ensureKubeControllerManagerTrustedCA(c.kubeClient.CoreV1(), recorder)
+	err = ensureKubeControllerManagerTrustedCA(ctx, c.kubeClient.CoreV1(), recorder)
 	if err != nil {
 		errors = append(errors, fmt.Errorf("%q: %v", "configmap/trusted-ca-bundle", err))
 	}
 
+	// The operator is not upgradeable if serving service CA addition to token secrets is enabled
+	// with the UnsupportedConfigOverride field
+	// EnableDeprecatedAndRemovedServiceCAKeyUntilNextRelease_ThisMakesClusterImpossibleToUpgrade.
+	//
+	// This should be removed in 4.6.
+	var upgradeableCondition operatorv1.OperatorCondition
+	if addServingServiceCAToTokenSecrets {
+		upgradeableCondition = operatorv1.OperatorCondition{
+			Type:    operatorv1.OperatorStatusTypeUpgradeable,
+			Status:  operatorv1.ConditionFalse,
+			Reason:  "AddServingServiceCAToTokenSecretsEnabled",
+			Message: "Disable the addition of the serving service ca to token secrets by removing EnableDeprecatedAndRemovedServiceCAKeyUntilNextRelease_ThisMakesClusterImpossibleToUpgrade from the operator's UnsupportedConfigOverrdies",
+		}
+	} else {
+		upgradeableCondition = operatorv1.OperatorCondition{
+			Type:   operatorv1.OperatorStatusTypeUpgradeable,
+			Status: operatorv1.ConditionTrue,
+		}
+	}
+	if _, _, err := v1helpers.UpdateStaticPodStatus(c.operatorClient, v1helpers.UpdateStaticPodConditionFn(upgradeableCondition)); err != nil {
+		return true, err
+	}
+
 	if len(errors) > 0 {
 		condition := operatorv1.OperatorCondition{
 			Type:    "TargetConfigControllerDegraded",
@@ -271,7 +288,7 @@ func createTargetConfigController(c TargetConfigController, recorder events.Reco
 
 func manageKubeControllerManagerConfig(client corev1client.ConfigMapsGetter, recorder events.Recorder, operatorSpec *operatorv1.StaticPodOperatorSpec) (*corev1.ConfigMap, bool, error) {
 	configMap := resourceread.ReadConfigMapV1OrDie(v411_00_assets.MustAsset("v4.1.0/kube-controller-manager/cm.yaml"))
-	defaultConfig := v411_00_assets.MustAsset("v4.1.0/kube-controller-manager/defaultconfig.yaml")
+	defaultConfig := v411_00_assets.MustAsset("v4.1.0/config/defaultconfig.yaml")
 	requiredConfigMap, _, err := resourcemerge.MergePrunedConfigMap(
 		&kubecontrolplanev1.KubeControllerManagerConfig{},
 		configMap,
@@ -303,12 +320,12 @@ func manageClusterPolicyControllerConfig(client corev1client.ConfigMapsGetter, r
 	return resourceapply.ApplyConfigMap(client, recorder, requiredConfigMap)
 }
 
-func ensureLocalhostRecoverySAToken(client corev1client.CoreV1Interface, recorder events.Recorder) error {
+func ensureLocalhostRecoverySAToken(ctx context.Context, client corev1client.CoreV1Interface, recorder events.Recorder) error {
 	requiredSA := resourceread.ReadServiceAccountV1OrDie(v411_00_assets.MustAsset("v4.1.0/kube-controller-manager/localhost-recovery-sa.yaml"))
 	requiredToken := resourceread.ReadSecretV1OrDie(v411_00_assets.MustAsset("v4.1.0/kube-controller-manager/localhost-recovery-token.yaml"))
 
 	saClient := client.ServiceAccounts(operatorclient.TargetNamespace)
-	serviceAccount, err := saClient.Get(requiredSA.Name, metav1.GetOptions{})
+	serviceAccount, err := saClient.Get(ctx, requiredSA.Name, metav1.GetOptions{})
 	if err != nil {
 		return err
 	}
@@ -316,7 +333,7 @@ func ensureLocalhostRecoverySAToken(client corev1client.CoreV1Interface, recorde
 	// The default token secrets get random names so we have created a custom secret
 	// to be populated with SA token so we have a stable name.
 	secretsClient := client.Secrets(operatorclient.TargetNamespace)
-	token, err := secretsClient.Get(requiredToken.Name, metav1.GetOptions{})
+	token, err := secretsClient.Get(ctx, requiredToken.Name, metav1.GetOptions{})
 	if err != nil {
 		return err
 	}
@@ -352,7 +369,7 @@ func ensureLocalhostRecoverySAToken(client corev1client.CoreV1Interface, recorde
 	return err
 }
 
-func managePod(configMapsGetter corev1client.ConfigMapsGetter, secretsGetter corev1client.SecretsGetter, recorder events.Recorder, operatorSpec *operatorv1.StaticPodOperatorSpec, imagePullSpec, operatorImagePullSpec, clusterPolicyControllerPullSpec string) (*corev1.ConfigMap, bool, error) {
+func managePod(ctx context.Context, configMapsGetter corev1client.ConfigMapsGetter, secretsGetter corev1client.SecretsGetter, recorder events.Recorder, operatorSpec *operatorv1.StaticPodOperatorSpec, imagePullSpec, operatorImagePullSpec, clusterPolicyControllerPullSpec string, addServingServiceCAToTokenSecrets bool) (*corev1.ConfigMap, bool, error) {
 	required := resourceread.ReadPodV1OrDie(v411_00_assets.MustAsset("v4.1.0/kube-controller-manager/pod.yaml"))
 	// TODO: If the image pull spec is not specified, the "${IMAGE}" will be used as value and the pod will fail to start.
 	images := map[string]string{
@@ -401,7 +418,7 @@ func managePod(configMapsGetter corev1client.ConfigMapsGetter, secretsGetter cor
 		containerArgsWithLoglevel[0] += fmt.Sprintf(" -v=%d", 2)
 	}
 
-	if _, err := secretsGetter.Secrets(required.Namespace).Get("serving-cert", metav1.GetOptions{}); err != nil && !apierrors.IsNotFound(err) {
+	if _, err := secretsGetter.Secrets(required.Namespace).Get(ctx, "serving-cert", metav1.GetOptions{}); err != nil && !apierrors.IsNotFound(err) {
 		return nil, false, err
 	} else if err == nil {
 		containerArgsWithLoglevel[0] += " --tls-cert-file=/etc/kubernetes/static-pod-resources/secrets/serving-cert/tls.crt"
@@ -423,6 +440,20 @@ func managePod(configMapsGetter corev1client.ConfigMapsGetter, secretsGetter cor
 		required.Spec.Containers[i].Env = append(container.Env, proxyEnvVars...)
 	}
 
+	if addServingServiceCAToTokenSecrets {
+		// Ensure the addition of serving service ca to token secrets by setting the environment
+		// variable that will enable the behavior in the controller manager.
+		for _, container := range required.Spec.Containers {
+			if container.Name == "kube-controller-manager" {
+				container.Env = append(container.Env, corev1.EnvVar{
+					Name:  "ADD_SERVICE_SERVING_CA_TO_TOKEN_SECRETS",
+					Value: "true",
+				})
+				break
+			}
+		}
+	}
+
 	configMap := resourceread.ReadConfigMapV1OrDie(v411_00_assets.MustAsset("v4.1.0/kube-controller-manager/pod-cm.yaml"))
 	configMap.Data["pod.yaml"] = resourceread.WritePodV1OrDie(required)
 	configMap.Data["forceRedeploymentReason"] = operatorSpec.ForceRedeploymentReason
@@ -438,7 +469,7 @@ func manageServiceAccountCABundle(lister corev1listers.ConfigMapLister, client c
 		resourcesynccontroller.ResourceLocation{Namespace: operatorclient.GlobalMachineSpecifiedConfigNamespace, Name: "kube-apiserver-server-ca"},
 		// include the ca bundle needed to recognize default
 		// certificates generated by cluster-ingress-operator
-		resourcesynccontroller.ResourceLocation{Namespace: operatorclient.GlobalMachineSpecifiedConfigNamespace, Name: "router-ca"},
+		resourcesynccontroller.ResourceLocation{Namespace: operatorclient.GlobalMachineSpecifiedConfigNamespace, Name: "default-ingress-cert"},
 	)
 	if err != nil {
 		return nil, false, err
@@ -461,7 +492,7 @@ func ManageCSRCABundle(lister corev1listers.ConfigMapLister, client corev1client
 	return resourceapply.ApplyConfigMap(client, recorder, requiredConfigMap)
 }
 
-func ManageCSRSigner(lister corev1listers.SecretLister, client corev1client.SecretsGetter, recorder events.Recorder) (*corev1.Secret, time.Duration, bool, error) {
+func ManageCSRSigner(ctx context.Context, lister corev1listers.SecretLister, client corev1client.SecretsGetter, recorder events.Recorder) (*corev1.Secret, time.Duration, bool, error) {
 	// get the certkey pair we will sign with. We're going to add the cert to a ca bundle so we can recognize the chain it signs back to the signer
 	csrSigner, err := lister.Secrets(operatorclient.OperatorNamespace).Get("csr-signer")
 	if apierrors.IsNotFound(err) {
@@ -481,7 +512,7 @@ func ManageCSRSigner(lister corev1listers.SecretLister, client corev1client.Secr
 	useAfter = useAfter.Add(5 * time.Minute)
 	now := time.Now()
 
-	oldSigner, err := client.Secrets(operatorclient.TargetNamespace).Get("csr-signer", metav1.GetOptions{})
+	oldSigner, err := client.Secrets(operatorclient.TargetNamespace).Get(ctx, "csr-signer", metav1.GetOptions{})
 	_, _, _, oldUseBefore, _ := extractSigner(oldSigner)
 	switch {
 	case apierrors.IsNotFound(err):
@@ -504,7 +535,7 @@ func ManageCSRSigner(lister corev1listers.SecretLister, client corev1client.Secr
 			"tls.crt": certBytes,
 			"tls.key": signingKey,
 		},
-		Type: corev1.SecretTypeOpaque,
+		Type: corev1.SecretTypeTLS,
 	}
 	secret, modified, err := resourceapply.ApplySecret(client, recorder, csrSigner)
 	return secret, 0, modified, err
@@ -541,7 +572,7 @@ func extractSigner(csrSigner *corev1.Secret) ([]byte, []byte, time.Time, time.Ti
 	return certBytes, signingKey, useAfter, useBefore, nil
 }
 
-func ManageCSRIntermediateCABundle(lister corev1listers.SecretLister, client corev1client.ConfigMapsGetter, recorder events.Recorder) (*corev1.ConfigMap, bool, error) {
+func ManageCSRIntermediateCABundle(ctx context.Context, lister corev1listers.SecretLister, client corev1client.ConfigMapsGetter, recorder events.Recorder) (*corev1.ConfigMap, bool, error) {
 	// get the certkey pair we will sign with. We're going to add the cert to a ca bundle so we can recognize the chain it signs back to the signer
 	csrSigner, err := lister.Secrets(operatorclient.OperatorNamespace).Get("csr-signer")
 	if apierrors.IsNotFound(err) {
@@ -563,7 +594,7 @@ func ManageCSRIntermediateCABundle(lister corev1listers.SecretLister, client cor
 		return nil, false, err
 	}
 
-	csrSignerCA, err := client.ConfigMaps(operatorclient.OperatorNamespace).Get("csr-signer-ca", metav1.GetOptions{})
+	csrSignerCA, err := client.ConfigMaps(operatorclient.OperatorNamespace).Get(ctx, "csr-signer-ca", metav1.GetOptions{})
 	if apierrors.IsNotFound(err) {
 		csrSignerCA = &corev1.ConfigMap{
 			ObjectMeta: metav1.ObjectMeta{Namespace: operatorclient.OperatorNamespace, Name: "csr-signer-ca"},
@@ -609,14 +640,14 @@ func ManageCSRIntermediateCABundle(lister corev1listers.SecretLister, client cor
 	return resourceapply.ApplyConfigMap(client, recorder, csrSignerCA)
 }
 
-func ensureKubeControllerManagerTrustedCA(client corev1client.CoreV1Interface, recorder events.Recorder) error {
+func ensureKubeControllerManagerTrustedCA(ctx context.Context, client corev1client.CoreV1Interface, recorder events.Recorder) error {
 	required := resourceread.ReadConfigMapV1OrDie(v411_00_assets.MustAsset("v4.1.0/kube-controller-manager/trusted-ca-cm.yaml"))
 	cmCLient := client.ConfigMaps(operatorclient.TargetNamespace)
 
-	cm, err := cmCLient.Get("trusted-ca-bundle", metav1.GetOptions{})
+	cm, err := cmCLient.Get(ctx, "trusted-ca-bundle", metav1.GetOptions{})
 	if err != nil {
 		if apierrors.IsNotFound(err) {
-			_, err = cmCLient.Create(required)
+			_, err = cmCLient.Create(ctx, required, metav1.CreateOptions{})
 		}
 		return err
 	}
@@ -624,7 +655,7 @@ func ensureKubeControllerManagerTrustedCA(client corev1client.CoreV1Interface, r
 	// update if modified by the user
 	if val, ok := cm.Labels["config.openshift.io/inject-trusted-cabundle"]; !ok || val != "true" {
 		cm.Labels["config.openshift.io/inject-trusted-cabundle"] = "true"
-		_, err = cmCLient.Update(cm)
+		_, err = cmCLient.Update(ctx, cm, metav1.UpdateOptions{})
 		return err
 	}
 
